import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.PriorityQueue;
import java.util.Stack;
import java.util.StringTokenizer;


/**
 * ICPC - CTU Open Contest 2021
 * Sample Solution: BFS that's not a BFS at all
 * 
 * O(n^(4/3)) solution
 * The common ancestor is searched in 3600-level and 60-level jumps.
 * 
 * @author Martin Kacer
 */
public class NonBfsCubicRootMK {
	StringTokenizer st = new StringTokenizer("");
	BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
	boolean hasNextToken() {
		try {
			while (!st.hasMoreTokens()) {
				String line = input.readLine();
				if (line == null) return false;
				st = new StringTokenizer(line);
			}
		} catch (IOException ex) { throw new RuntimeException(ex); }
		return true;
	}
	String nextToken() {
		return (!hasNextToken()) ? null : st.nextToken();
	}
	int nextInt() {
		return Integer.parseInt(nextToken());
	}
	public static void main(String[] args) {
		new NonBfsCubicRootMK().run();
	}
	
	static class Entry implements Comparable<Entry> {
		final int n, p, d;
		Entry(int nn, int pp, int dd) { n=nn; p=pp; d=dd; }
		@Override public int compareTo(Entry o) {
			return (d != o.d) ? Integer.compare(d, o.d) : (n != o.n) ? Integer.compare(n, o.n) : Integer.compare(p, o.p);
		}
	}
	
	static final int SKIP2 = 60, SKIP3 = SKIP2*SKIP2;
	int n, oi;
	List<List<Integer>> graph;
	int[] par, ord, p2, p3, dep;
	PriorityQueue<Entry> q = new PriorityQueue<>();
	Stack<Integer> path = new Stack<>();
	
	void dfs(int x) {
		dep[x] = path.size();
		p2[x] = path.size() < SKIP2 ? -1 : path.get(path.size() - SKIP2);
		p3[x] = path.size() < SKIP3 ? -1 : path.get(path.size() - SKIP3);
		path.add(x);
		for (int c : graph.get(x)) dfs(c);
		path.pop();
	}
	
	int findep(int a, int b) {
		while (dep[a] > dep[b]) a = par[a];
		while (dep[b] > dep[a]) b = par[b];
		while (p3[a] != p3[b]) { a = p3[a]; b = p3[b]; }
		while (p2[a] != p2[b]) { a = p2[a]; b = p2[b]; }
		while (a != b) { a = par[a]; b = par[b]; }
		return dep[a];
	}
	
	void run() {
		n = nextInt(); int m = nextInt();
		graph = new ArrayList<>(n);
		for (int i = 0; i < n; ++i) graph.add(new ArrayList<>());
		for (int i = 0; i < m; ++i) {
			int u = nextInt(), v = nextInt();
			graph.get(u).add(v);
			graph.get(v).add(u);
		}
		par = new int[n]; Arrays.fill(par, -2);
		dep = new int[n]; ord = new int[n];
		oi = 0;
		q.add(new Entry(0, -1, 0));
		while (!q.isEmpty()) {
			Entry e = q.poll();
			if (par[e.n] > -2) continue;
			par[e.n] = e.p; ord[oi++] = e.n;
			for (int o : graph.get(e.n)) if (par[o] == -2) q.add(new Entry(o, e.n, e.d + 1));
		}
		for (List<Integer> g : graph) g.clear();
		for (int i = 1; i < n; ++i) graph.get(par[i]).add(i);
		
		p2 = new int[n]; p3 = new int[n];
		dfs(0);
		int pos = 0; long sum = 0;
		for (int i = 0; i < n; ++i) {
			int nxt = ord[i], d = findep(pos, nxt);
			sum += dep[pos] - d + dep[nxt] - d;
			pos = nxt;
		}
		System.out.println(sum);
	}
}
